@@ -34,9 +34,11 @@ __docformat__ = "restructuredtext en"
 try:
     import readline
 except ImportError:
-    readline = None
+    # mypy: Incompatible types in assignment (expression has type "None",
+    # mypy: variable has type Module))
+    # conditional import
+    readline = None  # type: ignore
 import os
-import os.path as osp
 import sys
 from pdb import Pdb
 import inspect
@@ -46,12 +48,17 @@ from logilab.common.compat import StringIO
 try:
     from IPython import PyColorize
 except ImportError:
+
     def colorize(source, start_lineno, curlineno):
         """fallback colorize function"""
         return source
+
     def colorize_source(source):
         return source
+
+
 else:
+
     def colorize(source, start_lineno, curlineno):
         """colorize and annotate source with linenos
         (as in pdb's list command)
@@ -63,10 +70,10 @@ else:
         for index, line in enumerate(output.getvalue().splitlines()):
             lineno = index + start_lineno
             if lineno == curlineno:
-                annotated.append('%4s\t->\t%s' % (lineno, line))
+                annotated.append("%4s\t->\t%s" % (lineno, line))
             else:
-                annotated.append('%4s\t\t%s' % (lineno, line))
-        return '\n'.join(annotated)
+                annotated.append("%4s\t\t%s" % (lineno, line))
+        return "\n".join(annotated)
 
     def colorize_source(source):
         """colorize given source"""
@@ -83,7 +90,7 @@ def getsource(obj):
     or code object.  The source code is returned as a single string.  An
     IOError is raised if the source code cannot be retrieved."""
     lines, lnum = inspect.getsourcelines(obj)
-    return ''.join(lines), lnum
+    return "".join(lines), lnum
 
 
 ################################################################
@@ -95,6 +102,7 @@ class Debugger(Pdb):
     - overrides list command to search for current block instead
       of using 5 lines of context
     """
+
     def __init__(self, tcbk=None):
         Pdb.__init__(self)
         self.reset()
@@ -134,11 +142,10 @@ class Debugger(Pdb):
         """provide variable names completion for the ``p`` command"""
         namespace = dict(self.curframe.f_globals)
         namespace.update(self.curframe.f_locals)
-        if '.' in text:
+        if "." in text:
             return self.attr_matches(text, namespace)
         return [varname for varname in namespace if varname.startswith(text)]
 
-
     def attr_matches(self, text, namespace):
         """implementation coming from rlcompleter.Completer.attr_matches
         Compute matches when text contains a dot.
@@ -153,14 +160,15 @@ class Debugger(Pdb):
 
         """
         import re
+
         m = re.match(r"(\w+(\.\w+)*)\.(\w*)", text)
         if not m:
             return
         expr, attr = m.group(1, 3)
         object = eval(expr, namespace)
         words = dir(object)
-        if hasattr(object, '__class__'):
-            words.append('__class__')
+        if hasattr(object, "__class__"):
+            words.append("__class__")
             words = words + self.get_class_members(object.__class__)
         matches = []
         n = len(attr)
@@ -172,43 +180,45 @@ class Debugger(Pdb):
     def get_class_members(self, klass):
         """implementation coming from rlcompleter.get_class_members"""
         ret = dir(klass)
-        if hasattr(klass, '__bases__'):
+        if hasattr(klass, "__bases__"):
             for base in klass.__bases__:
                 ret = ret + self.get_class_members(base)
         return ret
 
-    ## specific / overridden commands
+    # specific / overridden commands
     def do_list(self, arg):
         """overrides default list command to display the surrounding block
         instead of 5 lines of context
         """
-        self.lastcmd = 'list'
+        self.lastcmd = "list"
         if not arg:
             try:
                 source, start_lineno = getsource(self.curframe)
-                print(colorize(''.join(source), start_lineno,
-                               self.curframe.f_lineno))
+                print(colorize("".join(source), start_lineno, self.curframe.f_lineno))
             except KeyboardInterrupt:
                 pass
             except IOError:
                 Pdb.do_list(self, arg)
         else:
             Pdb.do_list(self, arg)
+
     do_l = do_list
 
     def do_open(self, arg):
         """opens source file corresponding to the current stack level"""
         filename = self.curframe.f_code.co_filename
         lineno = self.curframe.f_lineno
-        cmd = 'emacsclient --no-wait +%s %s' % (lineno, filename)
+        cmd = "emacsclient --no-wait +%s %s" % (lineno, filename)
         os.system(cmd)
 
     do_o = do_open
 
+
 def pm():
     """use our custom debugger"""
     dbg = Debugger(sys.last_traceback)
     dbg.start()
 
+
 def set_trace():
     Debugger().set_trace(sys._getframe().f_back)
