@@ -66,6 +66,7 @@ def lazy_wraps(wrapped):
 class DeprecationWrapper(object):
     """proxy to print a warning on access to any attribute of the wrapped object
     """
+
     def __init__(self, proxied, msg=None, version=None):
         self._proxied = proxied
         self._msg = msg
@@ -76,14 +77,14 @@ class DeprecationWrapper(object):
         return getattr(self._proxied, attr)
 
     def __setattr__(self, attr, value):
-        if attr in ('_proxied', '_msg'):
+        if attr in ("_proxied", "_msg"):
             self.__dict__[attr] = value
         else:
             send_warning(self._msg, stacklevel=3, version=self.version)
             setattr(self._proxied, attr, value)
 
 
-def _get_package_name(number=2):
+def _get_module_name(number=1):
     """
     automagically try to determine the package name from which the warning has
     been triggered by loop other calling frames.
@@ -93,35 +94,33 @@ def _get_package_name(number=2):
 
     frame = sys._getframe()
 
-    for i in range(number):
+    for i in range(number + 1):
         if frame.f_back is None:
             break
 
         frame = frame.f_back
 
-    if frame.f_globals["__package__"] is not None:
+    if frame.f_globals["__package__"]:
         return frame.f_globals["__package__"]
 
     file_name = os.path.split(frame.f_globals["__file__"])[1]
 
     if file_name.endswith(".py"):
-        file_name = file_name[:-len(".py")]
+        file_name = file_name[: -len(".py")]
 
     return file_name
 
 
-def send_warning(reason, version=None, stacklevel=2):
+def send_warning(reason, version=None, stacklevel=2, module_name=None):
     """Display a deprecation message only if the version is older than the
     compatible version.
     """
-    module_name = _get_package_name(stacklevel + 1)
-
     if module_name and version:
-        reason = '[%s %s] %s' % (module_name, version, reason)
+        reason = "[%s %s] %s" % (module_name, version, reason)
     elif module_name:
-        reason = '[%s] %s' % (module_name, reason)
+        reason = "[%s] %s" % (module_name, reason)
     elif version:
-        reason = '[%s] %s' % (version, reason)
+        reason = "[%s] %s" % (version, reason)
 
     warn(reason, DeprecationWarning, stacklevel=stacklevel)
 
@@ -134,17 +133,24 @@ def callable_renamed(old_name, new_function, version=None):
 
     >>> old_function = renamed('old_function', new_function)
     >>> old_function()
-    sample.py:57: DeprecationWarning: old_function has been renamed and is deprecated, uses new_function instead
-    old_function()
+    sample.py:57: DeprecationWarning: old_function has been renamed and is deprecated, uses
+    new_function instead old_function()
     >>>
     """
+
     @lazy_wraps(new_function)
     def wrapped(*args, **kwargs):
-        send_warning((
-            f"{old_name} has been renamed and is deprecated, uses {new_function.__name__} "
-            f"instead"
-        ), stacklevel=3, version=version)
+        send_warning(
+            (
+                f"{old_name} has been renamed and is deprecated, uses {new_function.__name__} "
+                f"instead"
+            ),
+            stacklevel=3,
+            version=version,
+            module_name=new_function.__module__,
+        )
         return new_function(*args, **kwargs)
+
     return wrapped
 
 
@@ -159,16 +165,22 @@ def argument_removed(old_argument_name, version=None):
     ... def some_function(new):
     ...     return new
     >>> some_function(old=42)
-    sample.py:15: DeprecationWarning: argument old of callable some_function has been renamed and is deprecated, use keyword argument new instead
-      some_function(old=42)
+    sample.py:15: DeprecationWarning: argument old of callable some_function has been renamed and
+    is deprecated, use keyword argument new instead some_function(old=42)
     42
     """
+
     def _wrap(func):
         @lazy_wraps(func)
         def check_kwargs(*args, **kwargs):
             if old_argument_name in kwargs:
-                send_warning(f"argument {old_argument_name} of callable {func.__name__} has been "
-                             f"removed and is deprecated", stacklevel=3, version=version)
+                send_warning(
+                    f"argument {old_argument_name} of callable {func.__name__} has been "
+                    f"removed and is deprecated",
+                    stacklevel=3,
+                    version=version,
+                    module_name=func.__module__,
+                )
                 del kwargs[old_argument_name]
 
             return func(*args, **kwargs)
@@ -184,15 +196,17 @@ def callable_deprecated(reason=None, version=None, stacklevel=2):
     """Display a deprecation message only if the version is older than the
     compatible version.
     """
+
     def decorator(func):
         @lazy_wraps(func)
         def wrapped(*args, **kwargs):
             message = reason or 'The function "%s" is deprecated'
-            if '%s' in message:
+            if "%s" in message:
                 message %= func.__name__
 
-            send_warning(message, version, stacklevel + 1)
+            send_warning(message, version, stacklevel + 1, module_name=func.__module__)
             return func(*args, **kwargs)
+
         return wrapped
 
     return decorator
@@ -201,21 +215,23 @@ def callable_deprecated(reason=None, version=None, stacklevel=2):
 deprecated = callable_renamed(old_name="deprecated", new_function=callable_deprecated)
 
 
-def _generate_class_deprecated():
-    class _class_deprecated(type):
-        """metaclass to print a warning on instantiation of a deprecated class"""
+def class_deprecated(old_name, parent, class_dict):
+    class DeprecatedClass(*parent):
+        def __init__(self, *args, **kwargs):
+            msg = class_dict.get("__deprecation_warning__", f"{old_name} is deprecated")
+            send_warning(
+                msg,
+                stacklevel=class_dict.get("__deprecation_warning_stacklevel__", 3),
+                version=class_dict.get("__deprecation_warning_version__", None),
+                module_name=class_dict.get(
+                    "__deprecation_warning_module_name__", _get_module_name(1)
+                ),
+            )
+            super(DeprecatedClass, self).__init__(*args, **kwargs)
 
-        def __call__(cls, *args, **kwargs):
-            message = getattr(cls, "__deprecation_warning__",
-                              "%(cls)s is deprecated") % {'cls': cls.__name__}
-            send_warning(message, stacklevel=getattr(cls, "__deprecation_warning_stacklevel__", 4),
-                         version=getattr(cls, "__deprecation_warning_version__", None))
-            return type.__call__(cls, *args, **kwargs)
+    DeprecatedClass.__name__ = old_name
 
-    return _class_deprecated
-
-
-class_deprecated = _generate_class_deprecated()
+    return DeprecatedClass
 
 
 def attribute_renamed(old_name, new_name, version=None):
@@ -229,15 +245,18 @@ def attribute_renamed(old_name, new_name, version=None):
 
     >>> some_class = SomeClass()
     >>> print(some_class.old)
-    sample.py:15: DeprecationWarning: SomeClass.old has been renamed and is deprecated, use SomeClass.new instead
+    sample.py:15: DeprecationWarning: SomeClass.old has been renamed and is deprecated, use
+    SomeClass.new instead
       print(some_class.old)
     42
     >>> some_class.old = 43
-    sample.py:16: DeprecationWarning: SomeClass.old has been renamed and is deprecated, use SomeClass.new instead
+    sample.py:16: DeprecationWarning: SomeClass.old has been renamed and is deprecated, use
+    SomeClass.new instead
       some_class.old = 43
     >>> some_class.old == some_class.new
     True
     """
+
     def _class_wrap(klass):
         reason = (
             f"{klass.__name__}.{old_name} has been renamed and is deprecated, use "
@@ -245,15 +264,15 @@ def attribute_renamed(old_name, new_name, version=None):
         )
 
         def _get_old(self):
-            send_warning(reason, stacklevel=3, version=version)
+            send_warning(reason, stacklevel=3, version=version, module_name=klass.__module__)
             return getattr(self, new_name)
 
         def _set_old(self, value):
-            send_warning(reason, stacklevel=3, version=version)
+            send_warning(reason, stacklevel=3, version=version, module_name=klass.__module__)
             setattr(self, new_name, value)
 
         def _del_old(self):
-            send_warning(reason, stacklevel=3, version=version)
+            send_warning(reason, stacklevel=3, version=version, module_name=klass.__module__)
             delattr(self, new_name)
 
         setattr(klass, old_name, property(_get_old, _set_old, _del_old))
@@ -271,22 +290,30 @@ def argument_renamed(old_name, new_name, version=None):
     ... def some_function(new):
     ...     return new
     >>> some_function(old=42)
-    sample.py:15: DeprecationWarning: argument old of callable some_function has been renamed and is deprecated, use keyword argument new instead
+    sample.py:15: DeprecationWarning: argument old of callable some_function has been renamed and
+    is deprecated, use keyword argument new instead
       some_function(old=42)
     42
     """
+
     def _wrap(func):
         @lazy_wraps(func)
         def check_kwargs(*args, **kwargs):
             if old_name in kwargs and new_name in kwargs:
-                raise ValueError(f"argument {old_name} of callable {func.__name__} has been "
-                                 f"renamed to {new_name} but you are both using {old_name} and "
-                                 f"{new_name} has keyword arguments, only uses {new_name}")
+                raise ValueError(
+                    f"argument {old_name} of callable {func.__name__} has been "
+                    f"renamed to {new_name} but you are both using {old_name} and "
+                    f"{new_name} has keyword arguments, only uses {new_name}"
+                )
 
             if old_name in kwargs:
-                send_warning(f"argument {old_name} of callable {func.__name__} has been renamed "
-                             f"and is deprecated, use keyword argument {new_name} instead",
-                             stacklevel=3, version=version)
+                send_warning(
+                    f"argument {old_name} of callable {func.__name__} has been renamed "
+                    f"and is deprecated, use keyword argument {new_name} instead",
+                    stacklevel=3,
+                    version=version,
+                    module_name=func.__module__,
+                )
                 kwargs[new_name] = kwargs[old_name]
                 del kwargs[old_name]
 
@@ -315,7 +342,9 @@ def callable_moved(module_name, object_name, version=None, stacklevel=2):
     def callnew(*args, **kwargs):
         from logilab.common.modutils import load_module_from_name
 
-        send_warning(message, version=version, stacklevel=stacklevel + 1)
+        send_warning(
+            message, version=version, stacklevel=stacklevel + 1, module_name=_get_module_name(1)
+        )
 
         m = load_module_from_name(module_name)
         return getattr(m, object_name)(*args, **kwargs)
@@ -326,7 +355,7 @@ def callable_moved(module_name, object_name, version=None, stacklevel=2):
 moved = callable_renamed(old_name="moved", new_function=callable_moved)
 
 
-def class_renamed(old_name, new_class, message=None, version=None):
+def class_renamed(old_name, new_class, message=None, version=None, module_name=None):
     """automatically creates a class which fires a DeprecationWarning
     when instantiated.
 
@@ -338,27 +367,18 @@ def class_renamed(old_name, new_class, message=None, version=None):
     """
     class_dict = {}
     if message is None:
-        message = '%s is deprecated, use %s instead' % (old_name, new_class.__name__)
+        message = "%s is deprecated, use %s instead" % (old_name, new_class.__name__)
 
-    class_dict['__deprecation_warning__'] = message
-    class_dict['__deprecation_warning_version__'] = version
-    class_dict['__deprecation_warning_stacklevel__'] = 5
+    class_dict["__deprecation_warning__"] = message
+    class_dict["__deprecation_warning_version__"] = version
+    class_dict["__deprecation_warning_stacklevel__"] = 3
 
-    try:
-        return class_deprecated(old_name, (new_class,), class_dict)
-    except (NameError, TypeError):
-        # in case of conflicting metaclass situation
-        class DeprecatedClass(new_class):
-            def __init__(self, *args, **kwargs):
-                msg = class_dict.get("__deprecation_warning__", f"{old_name} is deprecated")
-                send_warning(
-                    msg,
-                    stacklevel=class_dict.get("__deprecation_warning_stacklevel__", 3),
-                    version=class_dict.get("__deprecation_warning_version__", None),
-                )
-                super(DeprecatedClass, self).__init__(*args, **kwargs)
+    if module_name:
+        class_dict["__deprecation_warning_module_name__"] = module_name
+    else:
+        class_dict["__deprecation_warning_module_name__"] = _get_module_name(1)
 
-        return DeprecatedClass
+    return class_deprecated(old_name, (new_class,), class_dict)
 
 
 def class_moved(new_class, old_name=None, message=None, version=None):
@@ -369,7 +389,12 @@ def class_moved(new_class, old_name=None, message=None, version=None):
         old_name = new_class.__name__
 
     if message is None:
-        message = 'class %s is now available as %s.%s' % (
-            old_name, new_class.__module__, new_class.__name__)
+        message = "class %s is now available as %s.%s" % (
+            old_name,
+            new_class.__module__,
+            new_class.__name__,
+        )
 
-    return class_renamed(old_name, new_class, message=message)
+    module_name = _get_module_name(1)
+
+    return class_renamed(old_name, new_class, message=message, module_name=module_name)
